# No Comment --- Comment any resource on the web!
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from flask import Blueprint, Response, request, url_for

from no_comment.application.use_cases import add_comment, view_comments
from no_comment.infrastructure.flask.utils import presenter_to_response
from no_comment.interfaces.to_http import as_html as html_presenters
from no_comment.interfaces.to_http import as_json as json_presenters
from no_comment.interfaces.to_http import as_xml as xml_presenters

from . import services

blueprint = Blueprint("comments", __name__)


@blueprint.get("/<string:stream_id>")
@blueprint.get("/<string:stream_id>.<string:format>")
@presenter_to_response
def index(stream_id: str, format: str = "html") -> Response:
    presenter = select_presenter_for_view_comments(
        stream_id,
        format,
        page=int(request.args.get("page", "1")),
        fragment=request.headers.get("Hx-Target", ""),
    )
    interactor = view_comments.Interactor(presenter, services.comments())
    rq = view_comments.Request(
        stream_id, request.args.get("url"), int(request.args.get("page", "1"))
    )
    interactor.execute(rq)
    return presenter


def select_presenter_for_view_comments(
    stream_id: str, format: str, page: int = 1, fragment: str = ""
) -> view_comments.Presenter:
    url_comments = url_for("comments.index", stream_id=stream_id, _external=True)
    url_feed = url_for(
        "comments.index", stream_id=stream_id, format=format, _external=True
    )
    description = "Description"
    author = "Author"
    PRESENTERS = {
        "html": html_presenters.Stream(stream_id, page=page, fragment=fragment),
        "atom": xml_presenters.StreamAsAtom(url_comments, stream_id, url_feed, author),
        "rss": xml_presenters.StreamAsRss(stream_id, description, url_comments),
        "json": json_presenters.StreamAsJsonFeed(),
    }
    return PRESENTERS.get(format, html_presenters.UnknownFormat())


@blueprint.post("/<string:stream_id>")
@presenter_to_response
def index_POST(stream_id: str) -> Response:
    presenter = html_presenters.AddComment(
        lambda: url_for("comments.index", stream_id=stream_id),
        fragment=request.headers.get("Hx-Target", ""),
    )
    interactor = add_comment.Interactor(
        presenter, services.comments(), services.calendar()
    )
    rq = add_comment.Request(
        stream_id, request.form.get("url", ""), request.form.get("text", "")
    )
    interactor.execute(rq)
    return presenter


@blueprint.get("/<string:stream_id>/<string:comment_id>")
@presenter_to_response
def comment(stream_id: str, comment_id: str) -> Response:
    return html_presenters.Comment()



# No Comment --- Comment any resource on the web!
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import json
from http import HTTPStatus as HTTP
from typing import Any

from no_comment.application.use_cases import add_comment, view_comments
from no_comment.domain.commenting import entities
from no_comment.interfaces.to_http import HttpPresenter


class JsonPresenter(HttpPresenter):
    def __init__(self) -> None:
        self.__status_code: HTTP = HTTP.OK
        self.__headers: dict[str, str] = {"Content-Type": "application/json"}
        self.__data: str = ""

    def status_code(self) -> int:
        return int(self.__status_code)

    def headers(self) -> dict[str, str]:
        return self.__headers

    def data(self) -> str:
        return self.__data


class StreamAsJsonFeed(view_comments.Presenter, JsonPresenter):
    def __init__(self) -> None:
        super().__init__()
        self.__data: dict[str, Any] = {
            "version": "https://jsonfeed.org/version/1.1",
            "title": "TITLE",
            "home_page_url": "https://example.org/",
            "feed_url": "https://example.org/feed.json",
            "items": [],
        }

    def data(self) -> str:
        return json.dumps(self.__data)

    def comment(self, comment: entities.Comment) -> None:
        self.__data["items"].append(
            {
                "id": "ID",
                "content_html": str(comment.text),
                "content_text": str(comment.text),
                "url": str(comment.url),
                "date_published": comment.created.to_isoformat(),
            }
        )


class AddComment(add_comment.Presenter, JsonPresenter):
    def __init__(self) -> None:
        super().__init__()
        self.__status_code = HTTP.NOT_IMPLEMENTED
        self.__data: dict[str, Any] = {}

    def status_code(self) -> int:
        return int(self.__status_code)

    def data(self) -> str:
        return json.dumps(self.__data)

    def bad_parameter(self, name: str, value: str) -> None:
        self.__status_code = HTTP.BAD_REQUEST
        self.__data = {"error": {name: value}}

    def comment_added(self, comment: entities.Comment) -> None:
        self.__status_code = HTTP.CREATED
        # TODO return comment's data!?


class ViewComments(view_comments.Presenter, JsonPresenter):
    def __init__(self) -> None:
        super().__init__()
        self.__status_code = HTTP.NO_CONTENT
        self.__data: list[dict[str, Any]] = []

    def status_code(self) -> int:
        return int(self.__status_code)

    def data(self) -> str:
        return json.dumps(self.__data)

    def comment(self, comment: entities.Comment) -> None:
        self.__status_code = HTTP.OK
        self.__data.append(
            {
                "text": str(comment.text),
                "url": str(comment.url),
                "created": comment.created.to_isoformat(),
            }
        )



# No Comment --- Comment any resource on the web!
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from abc import ABC, abstractmethod
from typing import Optional

from .entities import Comment
from .value_objects import StreamId, Url


class Comments(ABC):
    @abstractmethod
    def on_stream(
        self,
        stream_id: StreamId,
        /,
        *,
        latest_first: bool = True,
        page: Optional[int] = None,
        url: Optional[Url] = None,
    ) -> list[Comment]:
        ...

    @abstractmethod
    def add(self, stream_id: StreamId, comment: Comment) -> None:
        ...



# No Comment --- Comment any resource on the web!
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime, timezone

from no_comment.domain.commenting.entities import Comment
from no_comment.domain.commenting.value_objects import CommentId, DateAndTime, Text, Url


# FIXME `comment` is also defined as a fixture in `conftest.py`.
def comment() -> Comment:
    return Comment(
        CommentId.instanciate("00000000-0000-0000-0000-000000000000"),
        Url.instanciate("https://s.d.t/p"),
        Text.instanciate("The text of the comment."),
        DateAndTime.instanciate(datetime.now(timezone.utc)),
    )



# No Comment --- Comment any resource on the web!
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from abc import ABC, abstractmethod
from dataclasses import dataclass
from uuid import uuid4

from no_comment.domain.commenting.entities import Comment
from no_comment.domain.commenting.repositories import Comments
from no_comment.domain.commenting.services import Calendar
from no_comment.domain.commenting.value_objects import (
    CommentId,
    InvalidValue,
    StreamId,
    Text,
    Url,
)


@dataclass
class Request:
    stream_id: str
    url: str
    text: str


class Presenter(ABC):
    @abstractmethod
    def bad_parameter(self, name: str, value: str) -> None:
        ...

    @abstractmethod
    def comment_added(self, comment: Comment) -> None:
        ...


@dataclass
class Interactor:
    presenter: Presenter
    comments: Comments
    calendar: Calendar

    def execute(self, request: Request) -> None:
        try:
            url = Url.instanciate(request.url)
        except InvalidValue:
            return self.presenter.bad_parameter("url", "")

        try:
            text = Text.instanciate(request.text)
        except InvalidValue:
            return self.presenter.bad_parameter("text", "")

        # Comment should be built by a comment factory
        comment_id = CommentId.instanciate(str(uuid4()))
        stream_id = StreamId(request.stream_id)
        comment = Comment(comment_id, url, text, self.calendar.now())

        self.comments.add(stream_id, comment)
        self.presenter.comment_added(comment)



# No Comment --- Comment any resource on the web!
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from flask import current_app

from no_comment.domain.commenting.services import Calendar
from no_comment.infrastructure.file_system.repositories import Comments


def comments() -> Comments:
    return Comments(Path(current_app.config.get("DATA_DIR", "")))


def calendar() -> Calendar:
    # TODO: read timezone info from service and/or server configuration.
    return Calendar()



# No Comment --- Comment any resource on the web!
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from flask import Flask, get_flashed_messages, url_for
from werkzeug.middleware.proxy_fix import ProxyFix

import no_comment.interfaces.to_http.as_html as html_presenters
from no_comment import __version__

from .comments import blueprint as comments


def build_app(config: dict[str, str]) -> Flask:
    app = Flask(
        __name__,
        static_url_path="/resources",
        static_folder="./static/",
    )

    configure(app, config)
    register_blueprints(app)
    register_globals()

    return app


def configure(app: Flask, config: dict[str, str]) -> None:
    if config.get("PROXIED", ""):
        app.wsgi_app = ProxyFix(  # type: ignore[assignment]
            app.wsgi_app, x_for=1, x_proto=1, x_host=1, x_prefix=1
        )

    app.config.update(
        # SESSION_COOKIE_SECURE=True,
        SESSION_COOKIE_HTTPONLY=True,
        SESSION_COOKIE_SAMESITE="Lax",
    )
    app.config.update(config)


def register_blueprints(app: Flask) -> None:
    app.register_blueprint(comments, url_prefix="/")


def register_globals() -> None:
    html_presenters.register_jinja_global("version", __version__)
    html_presenters.register_jinja_global("url_for", url_for)
    html_presenters.register_jinja_global("get_flashed_messages", get_flashed_messages)



# No Comment --- Comment any resource on the web!
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from http import HTTPStatus as HTTP
from pathlib import Path
from typing import Any, Callable

from jinja2 import Environment, FileSystemLoader, select_autoescape
from jinja2_fragments import render_block

from no_comment.application.use_cases import add_comment, view_comments
from no_comment.domain.commenting import entities
from no_comment.domain.commenting import entities as e
from no_comment.interfaces.to_http import HttpPresenter

ENVIRONMENT = Environment(
    loader=FileSystemLoader([Path(__file__).parent / "templates"]),
    autoescape=select_autoescape(),
    extensions=["pypugjs.ext.jinja.PyPugJSExtension"],
)


def register_jinja_global(key: str, value: Any) -> None:
    ENVIRONMENT.globals[key] = value


class JinjaPresenter(HttpPresenter):
    def __init__(self, template: str, /, *, fragment: str = "", **context: Any) -> None:
        self.__status_code = HTTP.OK
        self.__headers = {"Content-Type": "text/html; charset=UTF-8"}
        self.__template = template
        self.__fragment = fragment
        self.__context = context
        self.__error = ""

    def status_code(self) -> int:
        return int(self.__status_code)

    def headers(self) -> dict[str, str]:
        return self.__headers

    def data(self) -> str:
        return self.render()

    def set_status_code(self, status_code: HTTP) -> None:
        self.__status_code = status_code

    def error(self, message: str, status_code: HTTP) -> None:
        self.__error = message
        self.__status_code = status_code

    def access_not_allowed(self, login: str) -> None:
        if login:
            self.__status_code = HTTP.FORBIDDEN
            self.__error = "You cannot access this page!"
        else:
            self.__status_code = HTTP.UNAUTHORIZED
            self.__error = "You must be logged in to access this page!"

    def render(self, **context: Any) -> str:
        if self.__error:
            return ENVIRONMENT.get_template("error.pug").render(
                **self.__context,
                **context,
                message=self.__error,
            )
        if self.__fragment:
            return render_block(  # type: ignore
                ENVIRONMENT,
                self.__template,
                self.__fragment,
                **self.__context,
                **context,
            )
        return ENVIRONMENT.get_template(self.__template).render(
            **self.__context, **context
        )


class PugPresenter(JinjaPresenter):
    def __init__(self, template: str, **context: Any) -> None:
        super().__init__(template + ".pug", **context)


class UnknownFormat(view_comments.Presenter, PugPresenter):
    def __init__(self) -> None:
        super().__init__("unknown_format")
        self.set_status_code(HTTP.NOT_FOUND)

    def comment(self, comment: entities.Comment) -> None:
        pass


class Stream(view_comments.Presenter, PugPresenter):
    def __init__(self, stream_id: str, /, *, page: int = 1, fragment: str = "") -> None:
        super().__init__("stream", fragment=fragment)
        self.__context: dict[str, Any] = {"stream_id": stream_id, "comments": []}
        self.__set_page_urls(page)

    def __set_page_urls(self, page: int) -> None:
        if page > 1:
            self.__context["previous_page_url"] = f"?page={page - 1}"
        # FIXME how to know it's the last page?!
        self.__context["next_page_url"] = f"?page={page + 1}"

    def comment(self, comment: entities.Comment) -> None:
        self.__context["comments"].append(
            {
                "url": str(comment.url),
                "text": str(comment.text),
                "created": comment.created.format("%Y-%m-%d"),
            },
        )

    def render(self, **context: Any) -> str:
        return super().render(**self.__context, **context)


class Comment(PugPresenter):
    def __init__(self) -> None:
        super().__init__("comment")


class AddComment(PugPresenter, add_comment.Presenter):
    def __init__(self, next_url: Callable[[], str], /, *, fragment: str = "") -> None:
        super().__init__("stream", fragment=fragment)
        self.__next_url = next_url
        self.__fragment = fragment
        self.__comment_id = ""
        self.__error = ""

    def status_code(self) -> int:
        if self.__fragment:
            return HTTP.OK
        return HTTP.SEE_OTHER

    def headers(self) -> dict[str, str]:
        if self.__fragment and self.__comment_id:
            return {"HX-Trigger": "commentAdded"}
        return {"Location": self.__next_url()}

    def data(self) -> str:
        if self.__fragment:
            return super().render(comment_id=self.__comment_id, error=self.__error)
        return ""

    def bad_parameter(self, name: str, value: str) -> None:
        self.__error = f"Missing {name}!"

    def comment_added(self, comment: e.Comment) -> None:
        self.__comment_id = str(comment.id)



# No Comment --- Comment any resource on the web!
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Optional

from no_comment.domain.commenting.entities import Comment
from no_comment.domain.commenting.repositories import Comments
from no_comment.domain.commenting.value_objects import StreamId, Url


@dataclass
class Request:
    stream_id: str
    url: Optional[str] = None
    page: Optional[int] = None


class Presenter(ABC):
    @abstractmethod
    def comment(self, comment: Comment) -> None:
        ...


@dataclass
class Interactor:
    presenter: Presenter
    comments: Comments

    def execute(self, request: Request) -> None:
        for comment in self.comments.on_stream(
            StreamId(request.stream_id),
            url=Url(request.url) if request.url else None,
            page=request.page,
            latest_first=True,
        ):
            self.presenter.comment(comment)



# No Comment --- Comment any resource on the web!
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime, timezone
from typing import Type, TypeVar
from urllib.parse import urlparse

from bl3d import DateAndTime as DT
from bl3d import InvalidValue as BaseInvalidValue
from bl3d import String

TypeText = TypeVar("TypeText", bound="Text")
TypeUrl = TypeVar("TypeUrl", bound="Url")


class InvalidValue(BaseInvalidValue):
    pass


class DateAndTime(DT):
    # TODO: use the one available in bl3d > 0.3.0
    @classmethod
    def from_isoformat(cls, isoformat: str) -> "DateAndTime":
        dt = datetime.fromisoformat(isoformat)
        if not dt.tzinfo:
            raise InvalidValue(cls.__name__, "Must define a timezone.")
        return cls.instanciate(dt.astimezone(timezone.utc))

    def __lt__(self, other: object) -> bool:
        if not isinstance(other, DateAndTime):
            raise NotImplementedError()

        return self.to_datetime() < other.to_datetime()

    # TODO: use the one available in bl3d > 0.3.0
    def to_isoformat(self, /, *, with_microseconds: bool = True) -> str:
        dt = self.to_datetime()
        if not with_microseconds:
            dt = dt.replace(microsecond=0)
        return dt.isoformat()


class UniversalUniqueIdentifier(String):
    MIN: int = 36
    MAX: int = 36


class StreamId(String):
    pass


class CommentId(UniversalUniqueIdentifier):
    pass


class Text(String):
    @classmethod
    def instanciate(cls: Type[TypeText], value: str) -> TypeText:
        _ = String.instanciate(value)

        if not value.strip():
            raise InvalidValue(cls.__name__, "Text cannot be empty!")

        return cls(value)

    def __repr__(self) -> str:
        text = str(self).replace("'", "\\'")
        if len(text) > 11:
            text = text[:5] + "…" + text[-5:]
        return f"Text('{text}')"


class Url(String):
    @classmethod
    def instanciate(cls: Type[TypeUrl], value: str) -> TypeUrl:
        _ = String.instanciate(value)

        url = urlparse(value)
        if not url.scheme or not url.netloc:
            raise InvalidValue(cls.__name__, f"Invalid URL! [{value}]")

        return cls(value)

    def __repr__(self) -> str:
        return f"Url('{str(self)}')"



# No Comment --- Comment any resource on the web!
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime, timezone
from email.utils import format_datetime
from http import HTTPStatus as HTTP

from no_comment.application.use_cases import view_comments
from no_comment.domain.commenting import entities
from no_comment.interfaces.to_http import HttpPresenter


class XmlPresenter(HttpPresenter):
    def __init__(self, content_type: str = "application/xml") -> None:
        self.__status_code: HTTP = HTTP.OK
        self.__headers: dict[str, str] = {"Content-Type": content_type}
        self.__data: str = ""

    def status_code(self) -> int:
        return int(self.__status_code)

    def headers(self) -> dict[str, str]:
        return self.__headers

    def data(self) -> str:
        return self.__data


class StreamAsAtom(view_comments.Presenter, XmlPresenter):
    def __init__(self, id: str, title: str, url: str, author: str) -> None:
        super().__init__("application/atom+xml;charset=utf-8")
        self.__id = id
        self.__title = title
        self.__url = url
        self.__author = author
        self.__entries: str = ""
        self.__updated_on: str = "1970-01-01T00:00:00+00:00"

    def data(self) -> str:
        return f"""<?xml version="1.0" encoding="utf-8"?>
<feed xmlns="http://www.w3.org/2005/Atom">
    <id>{self.__id}</id>
    <title>{self.__title}</title>
    <link rel="self" type="application/atom+xml" href="{self.__url}" />
    <link rel="alternate" type="text/html" href="{self.__id}" />
    <updated>{self.__updated_on}</updated>
    <author>
        <name>{self.__author}</name>
    </author>
    <link href="http://creativecommons.org/licenses/by-sa/1.0/legalcode" rel="license" />
    {self.__entries}
</feed>"""

    def comment(self, comment: entities.Comment) -> None:
        updated_on = comment.created.to_isoformat(with_microseconds=False)
        if updated_on > self.__updated_on:
            self.__updated_on = updated_on
        self.__entries += f"""
    <entry>
        <id>urn:uuid:{comment.id}</id>
        <title>{comment_title(comment)}</title>
        <link rel="via" href="{comment.url}" />
        <updated>{updated_on}</updated>
        <content>{comment.text}</content>
    </entry>"""


class StreamAsRss(view_comments.Presenter, XmlPresenter):
    def __init__(self, title: str, description: str, url: str) -> None:
        super().__init__("application/rss+xml;charset=utf-8")
        self.__title = title
        self.__description = description
        self.__url = url
        self.__items: str = ""
        self.__updated_on = datetime(1970, 1, 1, 0, 0, 0).astimezone(timezone.utc)

    def data(self) -> str:
        return f"""<?xml version="1.0" encoding="UTF-8" ?>
<rss version="2.0" xmlns:atom="http://www.w3.org/2005/Atom">
    <channel>
        <title>{self.__title}</title>
        <description>{self.__description}</description>
        <lastBuildDate>{format_datetime(self.__updated_on)}</lastBuildDate>
        <link>{self.__url}</link>
        <atom:link href="{self.__url}.rss" rel="self" type="application/rss+xml" />
        {self.__items}
    </channel>
</rss>"""

    def comment(self, comment: entities.Comment) -> None:
        updated_on = comment.created.to_datetime()
        if updated_on > self.__updated_on:
            self.__updated_on = updated_on
        self.__items += f"""
        <item>
            <title>{comment_title(comment)}</title>
            <link>{comment.url}</link>
            <description>{comment.text}</description>
            <pubDate>{format_datetime(updated_on)}</pubDate>
            <guid isPermaLink="false">urn:uuid:{comment.id}</guid>
        </item>"""


def comment_title(comment: entities.Comment) -> str:
    TITLE_MAX_LENGTH = 30

    title = str(comment.text).split("\n")[0]
    if len(title) > TITLE_MAX_LENGTH:
        return title[0 : TITLE_MAX_LENGTH - 1] + "…"
    return title



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['no_comment',
 'no_comment.application',
 'no_comment.application.use_cases',
 'no_comment.configuration',
 'no_comment.domain',
 'no_comment.domain.commenting',
 'no_comment.infrastructure',
 'no_comment.infrastructure.file_system',
 'no_comment.infrastructure.flask',
 'no_comment.interfaces',
 'no_comment.interfaces.to_http',
 'no_comment.interfaces.to_http.as_html']

package_data = \
{'': ['*'],
 'no_comment.infrastructure.flask': ['static/css/*',
                                     'static/img/*',
                                     'static/js/*'],
 'no_comment.interfaces.to_http.as_html': ['templates/*', 'templates/mixins/*']}

install_requires = \
['Flask>=2.2.2,<3.0.0',
 'bl3d>=0.3.0,<0.4.0',
 'jinja2-fragments>=0.3.0,<0.4.0',
 'pypugjs>=5.9.12,<6.0.0']

setup_kwargs = {
    'name': 'no-comment',
    'version': '0.1.0a8',
    'description': 'Comment any resource on the web!',
    'long_description': '# No Comment\n\nComment any resource on the web!\n',
    'author': 'Tanguy Le Carrour',
    'author_email': 'tanguy@bioneland.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



# No Comment --- Comment any resource on the web!
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import json
from pathlib import Path
from typing import Optional

from no_comment.domain.commenting.entities import Comment
from no_comment.domain.commenting.repositories import Comments as InterfaceComments
from no_comment.domain.commenting.value_objects import (
    CommentId,
    DateAndTime,
    StreamId,
    Text,
    Url,
)


class Comments(InterfaceComments):
    PAGE_SIZE: int = 10

    def __init__(self, data_dir: Path) -> None:
        self.__data_dir = data_dir

    def on_stream(
        self,
        stream_id: StreamId,
        /,
        *,
        latest_first: bool = True,
        page: Optional[int] = None,
        url: Optional[Url] = None,
    ) -> list[Comment]:
        path = self.__path_for(stream_id)
        if not path.exists():
            return []
        comments = [
            # TODO: decide what to do with bad data.
            Comment(
                CommentId(data.get("id", "")),
                Url(data.get("url", "")),
                Text(data.get("text", "")),
                DateAndTime.from_isoformat(
                    data.get("created", "1971-01-01T00:00:00+00:00")
                ),
            )
            for data in [json.loads(line) for line in path.read_text().splitlines()]
        ]
        comments = sorted(comments, key=lambda c: c.created, reverse=latest_first)

        first = ((page or 1) - 1) * Comments.PAGE_SIZE
        number = Comments.PAGE_SIZE
        return comments[first:][:number]

    def __path_for(self, stream_id: StreamId) -> Path:
        return self.__data_dir / f"{stream_id}.jsons"

    def add(self, stream_id: StreamId, comment: Comment) -> None:
        data = {
            "id": str(comment.id),
            "url": str(comment.url),
            "text": str(comment.text),
            "created": comment.created.to_isoformat(),
        }
        path = self.__path_for(stream_id)
        with path.open("a") as fh:
            fh.write(json.dumps(data))
            fh.write("\n")
